<?php

declare(strict_types=1);

namespace Drupal\group_relationship_inheritance\Plugin\Field\FieldType;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Field\Plugin\Field\FieldType\BooleanItem;
use Drupal\group\Entity\GroupRelationship;

/**
 * Defines the 'group_relationship_inheritance' field type.
 *
 * @FieldType(
 *   id = "group_relationship_inheritance",
 *   label = @Translation("Group relationship inheritance"),
 *   description = @Translation("This is a computed field to set referenced entities as group related content."),
 *   default_widget = "group_relationship_inheritance_checkbox",
 *   default_formatter = "boolean",
 *   no_ui = TRUE,
 * )
 */
final class GroupRelationshipInheritanceItem extends BooleanItem {

  /**
   * {@inheritdoc}
   *
   * We need to override the presave to avoid saving without the host entity id
   * generated.
   */
  public function preSave(): void {
  }

  /**
   * {@inheritdoc}
   */
  public function postSave($update): void {
    // Only operate on forms with this property defined as true.
    // By default it will be set to FALSE and turned to TRUE in
    // form_alter so if this must be done in something like jsonapi
    // it needs to be defined as true in a property of the request.
    if ($this->values['value']) {

      $host_entity = $this->getEntity();

      $relatable_groups = [];
      if ($context_group = \Drupal::routeMatch()->getParameter('group')) {
        $relatable_groups[$context_group->id()] = $context_group;
      }
      foreach (GroupRelationship::loadByEntity($host_entity) as $relationship_id => $relationship) {
        $relatable_groups[$relationship->getGroupId()] = $relationship->getGroup();
      }

      if (!empty($relatable_groups)) {
        /** @var \Drupal\group\Plugin\Group\Relation\GroupRelationTypeManagerInterface **/
        // @todo DI
        $plugin_manager = \Drupal::service('group_relation_type.manager');

        foreach ($host_entity->referencedEntities() as $referenced_entity) {
          // May be other kind of entities to reference here (media, terms...)
          if ($referenced_entity->getEntityTypeId() === 'node') {
            $plugins = $plugin_manager->getPluginIdsByEntityTypeId($referenced_entity->getEntityTypeId());

            $match_plugin = array_filter($plugin_manager->getDefinitions(), fn ($i) => $i->getEntityBundle() === $referenced_entity->bundle());
            if (!empty($match_plugin)) {
              $this->attachRelationshipToReferenced($referenced_entity, array_key_first($match_plugin), $relatable_groups);
            }
          }
        }
      }
      parent::postSave($update);
    }
  }

  /**
   * Try to attach relationship to an entity and to relatable_groups.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $referenced_entity
   *   The referenced entity that will be related to the relatable groups.
   * @param string $plugin_id
   *   The grouprelationship plugin id that will be used.
   * @param array $relatable_groups
   *   The relatable groups.
   */
  protected function attachRelationshipToReferenced(ContentEntityInterface $referenced_entity, $plugin_id, array $relatable_groups): void {
    foreach ($relatable_groups as $relatable_group) {
      // @todo DI
      $account = \Drupal::currentUser()->getAccount();
      if ($relatable_group->hasPermission("create $plugin_id entity", $account)) {
        // @todo DI
        $duplicated_relationships = \Drupal::service('entity_type.manager')
          ->getStorage('group_relationship')->loadByEntityAndGroup($referenced_entity, $relatable_group, $plugin_id);
        if (empty($duplicated_relationships)) {
          $relatable_group->addRelationship($referenced_entity, $plugin_id);
        }
      }
      else {
        \Drupal::messenger()->addMessage($this->t('Could not relate the @plugin_id : @entity_label with group @group, some content may not appear in group content.', [
          '@plugin_id' => $plugin_id,
          '@entity_label' => $referenced_entity->label(),
          '@group' => $relatable_group->label(),
        ]), 'warning', TRUE);
      }
    }
  }

}
